## Sid Meier's Civilization 4
## Copyright Firaxis Games 2005
##
## Sevopedia 
##   sevotastic.blogspot.com
##   sevotastic@yahoo.com
##


from CvPythonExtensions import *
import CvUtil
import CvPediaScreen
import ScreenInput
import CvScreenEnums
import string

# globals
gc = CyGlobalContext()
ArtFileMgr = CyArtFileMgr()
localText = CyTranslator()

class CvPediaTech(CvPediaScreen.CvPediaScreen):
	"Civilopedia Screen for Techs"

	def __init__(self, main):
		self.iTech = -1
		self.top = main
		
		self.MAX_WIDTH = 1010
		self.MAX_HEIGHT = 700

		self.GAP_WIDTH = 15
		self.GAP_HEIGHT = 2
		self.MARGIN_WIDTH = 8
		self.MARGIN_HEIGHT = 6
	
		self.X_TECH_PANE = self.top.X_PEDIA_PAGE + self.GAP_WIDTH
		self.Y_TECH_PANE = 55
		self.W_TECH_PANE = 250
		self.H_TECH_PANE = 112

		self.X_ICON = self.X_TECH_PANE + self.MARGIN_WIDTH
		self.Y_ICON = self.Y_TECH_PANE + self.MARGIN_WIDTH
		self.W_ICON = 96
		self.H_ICON = 96
		self.ICON_SIZE = 64

		self.X_COST = self.X_TECH_PANE + 100
		self.Y_COST = self.Y_TECH_PANE + 35
		
		self.BUTTON_SIZE = 64

		self.X_PREREQ_PANE = self.X_TECH_PANE + self.W_TECH_PANE + self.GAP_WIDTH
		self.Y_PREREQ_PANE = self.Y_TECH_PANE
		self.W_PREREQ_PANE = self.MAX_WIDTH - self.X_PREREQ_PANE
		self.H_PREREQ_PANE = self.H_TECH_PANE

		self.X_SPECIAL_PANE = self.X_TECH_PANE
		self.Y_SPECIAL_PANE = self.Y_TECH_PANE + self.H_TECH_PANE + self.GAP_HEIGHT
		self.W_SPECIAL_PANE = self.W_TECH_PANE

		self.X_LEADS_TO_PANE = self.X_TECH_PANE + self.W_TECH_PANE + self.GAP_WIDTH
		self.Y_LEADS_TO_PANE = self.Y_SPECIAL_PANE
		self.W_LEADS_TO_PANE = self.MAX_WIDTH - self.X_LEADS_TO_PANE
		self.H_LEADS_TO_PANE = 116

		self.X_UNIT_PANE = self.X_LEADS_TO_PANE
		self.Y_UNIT_PANE = self.Y_LEADS_TO_PANE + self.H_LEADS_TO_PANE + self.GAP_HEIGHT
		self.W_UNIT_PANE = self.MAX_WIDTH - self.X_UNIT_PANE
		self.H_UNIT_PANE = self.H_LEADS_TO_PANE

		self.H_SPECIAL_PANE = self.H_LEADS_TO_PANE + self.H_UNIT_PANE + self.GAP_HEIGHT

		self.X_BUILDING_PANE = self.X_UNIT_PANE 
		self.Y_BUILDING_PANE = self.Y_UNIT_PANE + self.H_UNIT_PANE + self.GAP_HEIGHT
		self.W_BUILDING_PANE = self.MAX_WIDTH - self.X_BUILDING_PANE
		self.H_BUILDING_PANE = self.H_LEADS_TO_PANE
		
		self.X_OTHER_PANE = self.X_TECH_PANE 
		self.Y_OTHER_PANE = self.Y_SPECIAL_PANE + self.H_SPECIAL_PANE + self.GAP_HEIGHT
		self.W_OTHER_PANE = self.W_TECH_PANE
		self.H_OTHER_PANE = self.H_BUILDING_PANE

		self.X_QUOTE_PANE = self.X_TECH_PANE
		self.Y_QUOTE_PANE = self.Y_OTHER_PANE + self.H_OTHER_PANE + self.MARGIN_HEIGHT
		self.W_QUOTE_PANE = self.MAX_WIDTH - self.X_QUOTE_PANE
		self.H_QUOTE_PANE = self.MAX_HEIGHT - self.Y_QUOTE_PANE
		
		self.X_QUOTE = self.X_QUOTE_PANE + self.MARGIN_WIDTH
		self.Y_QUOTE = self.Y_QUOTE_PANE + self.MARGIN_HEIGHT
		self.W_QUOTE = self.W_QUOTE_PANE - 2 * self.MARGIN_WIDTH
		self.H_QUOTE = self.H_QUOTE_PANE - 2 * self.MARGIN_HEIGHT

	def interfaceScreen(self, iTech):	

		self.iTech = iTech
		self.techInfo = gc.getTechInfo(iTech)
	
		self.top.deleteAllWidgets()						
							
		screen = self.top.getScreen()
		
		bNotActive = (not screen.isActive())
		if bNotActive:
			self.top.setPediaCommonWidgets()

		# Header...
		szHeader = u"<font=4b>" + self.techInfo.getDescription().upper() + u"</font>"
		screen.setText(self.top.getNextWidgetName(), "Background", szHeader, CvUtil.FONT_CENTER_JUSTIFY, self.top.X_SCREEN, self.top.Y_TITLE, 0, FontTypes.TITLE_FONT, WidgetTypes.WIDGET_PEDIA_DESCRIPTION, CivilopediaPageTypes.CIVILOPEDIA_PAGE_TECH, iTech)
		screen.setImageButton(self.top.getNextWidgetName(), ArtFileMgr.getInterfaceArtInfo("INTERFACE_GENERAL_CIVILOPEDIA_ICON").getPath(), self.top.X_EXIT, self.top.Y_TITLE, 32, 32, WidgetTypes.WIDGET_PEDIA_DESCRIPTION,  CivilopediaPageTypes.CIVILOPEDIA_PAGE_TECH, iTech)
		
		# Top
		screen.setText(self.top.getNextWidgetName(), "Background", self.top.MENU_TEXT, CvUtil.FONT_LEFT_JUSTIFY, self.top.X_MENU, self.top.Y_MENU, 0, FontTypes.TITLE_FONT, WidgetTypes.WIDGET_PEDIA_MAIN, CivilopediaPageTypes.CIVILOPEDIA_PAGE_TECH, -1)

		if self.top.iLastScreen	!= CvScreenEnums.PEDIA_TECH or bNotActive:		
			if self.top.iLastScreen != CvScreenEnums.PEDIA_MAIN:
				self.placeLinks()
			self.top.iLastScreen = CvScreenEnums.PEDIA_TECH

		screen.addPanel( self.top.getNextWidgetName(), "", "", False, False, self.X_TECH_PANE, self.Y_TECH_PANE, self.W_TECH_PANE, self.H_TECH_PANE, PanelStyles.PANEL_STYLE_BLUE50)

		# Icon
		screen.addPanel(self.top.getNextWidgetName(), "", "", false, false, self.X_ICON, self.Y_ICON, self.W_ICON, self.H_ICON, PanelStyles.PANEL_STYLE_MAIN)
		screen.addDDSGFC(self.top.getNextWidgetName(), self.techInfo.getButton(), self.X_ICON + self.W_ICON/2 - self.ICON_SIZE/2, self.Y_ICON + self.H_ICON/2 - self.ICON_SIZE/2, self.ICON_SIZE, self.ICON_SIZE, WidgetTypes.WIDGET_GENERAL, -1, -1 )
		
		szCostId = self.top.getNextWidgetName()
		if self.top.iActivePlayer == -1:
			szCostText = localText.getText("TXT_KEY_PEDIA_COST", ( gc.getTechInfo(iTech).getResearchCost(), ) ) + u"%c" % (gc.getCommerceInfo(CommerceTypes.COMMERCE_RESEARCH).getChar())
		else:
			szCostText = localText.getText("TXT_KEY_PEDIA_COST", ( gc.getTeam(gc.getGame().getActiveTeam()).getResearchCost(iTech), ) ) + u"%c" % (gc.getCommerceInfo(CommerceTypes.COMMERCE_RESEARCH).getChar())
		screen.setLabel(szCostId, "Background", u"<font=3>" + szCostText.upper() + u"</font>", CvUtil.FONT_LEFT_JUSTIFY, self.X_COST + 25, self.Y_COST, 0, FontTypes.TITLE_FONT, WidgetTypes.WIDGET_GENERAL, -1, -1)
						
		# Place Required techs
		self.placePrereqs()

		# Place Allowing techs
		self.placeLeadsTo()
		
		# Place Units
		self.placeUnits()
		
		# Place buildings
		self.placeBuildings()
		
		# Place improvement
		self.placeOther()
		
		# Place the Special abilities block
		self.placeSpecial()

		# Place the quote for this technology		
		self.placeQuote()
			

	def placeLeadsTo(self):

		screen = self.top.getScreen()
		szLeadsTo = localText.getText("TXT_KEY_PEDIA_LEADS_TO", ())
		panelName = self.top.getNextWidgetName()
		screen.addPanel(panelName, szLeadsTo, "", false, true, self.X_LEADS_TO_PANE, self.Y_LEADS_TO_PANE, self.W_LEADS_TO_PANE, self.H_LEADS_TO_PANE, PanelStyles.PANEL_STYLE_BLUE50 )
		screen.attachLabel(panelName, "", "  ")

		for j in range(gc.getNumTechInfos()):
			for k in range(gc.getDefineINT("NUM_OR_TECH_PREREQS")):
				iPrereq = gc.getTechInfo(j).getPrereqOrTechs(k)
				if (iPrereq == self.iTech):
					screen.attachImageButton( panelName, "", gc.getTechInfo(j).getButton(), GenericButtonSizes.BUTTON_SIZE_CUSTOM, WidgetTypes.WIDGET_PEDIA_JUMP_TO_DERIVED_TECH, j, self.iTech, False )
			for k in range(gc.getDefineINT("NUM_AND_TECH_PREREQS")):
				iPrereq = gc.getTechInfo(j).getPrereqAndTechs(k)
				if (iPrereq == self.iTech):
					screen.attachImageButton( panelName, "", gc.getTechInfo(j).getButton(), GenericButtonSizes.BUTTON_SIZE_CUSTOM, WidgetTypes.WIDGET_PEDIA_JUMP_TO_DERIVED_TECH, j, self.iTech, False )

	def placePrereqs(self):
		
		screen = self.top.getScreen()
		szRequires = localText.getText("TXT_KEY_PEDIA_REQUIRES", ())
		panelName = self.top.getNextWidgetName()
		screen.addPanel( panelName, szRequires, "", false, true, self.X_PREREQ_PANE, self.Y_PREREQ_PANE, self.W_PREREQ_PANE, self.H_PREREQ_PANE, PanelStyles.PANEL_STYLE_BLUE50 )
		screen.attachLabel(panelName, "", "  ")
		
		bFirst = True
		for j in range(gc.getDefineINT("NUM_AND_TECH_PREREQS")):
			eTech = self.techInfo.getPrereqAndTechs(j)
			if (eTech > -1):
				if (not bFirst):
					screen.attachLabel(panelName, "", localText.getText("TXT_KEY_AND", ()))
				else:
					bFirst = False
				screen.attachImageButton( panelName, "", gc.getTechInfo(eTech).getButton(), GenericButtonSizes.BUTTON_SIZE_CUSTOM, WidgetTypes.WIDGET_PEDIA_JUMP_TO_REQUIRED_TECH, eTech, j, False )
					
		# count the number of OR techs
		nOrTechs = 0
		for j in range(gc.getDefineINT("NUM_OR_TECH_PREREQS")):
			if (self.techInfo.getPrereqOrTechs(j) > -1):
				nOrTechs += 1
				
		szLeftDelimeter = ""
		szRightDelimeter = ""
		#  Display a bracket if we have more than one OR tech and at least one AND tech
		if (not bFirst):
			if (nOrTechs > 1):
				szLeftDelimeter = localText.getText("TXT_KEY_AND", ()) + "( "
				szRightDelimeter = " ) "
			elif (nOrTechs > 0):
				szLeftDelimeter = localText.getText("TXT_KEY_AND", ())
			else:
				return

		if len(szLeftDelimeter) > 0:
			screen.attachLabel(panelName, "", szLeftDelimeter)
			
		bFirst = True
		for j in range(gc.getDefineINT("NUM_OR_TECH_PREREQS")):
			eTech = self.techInfo.getPrereqOrTechs(j)
			if (eTech > -1):
				if (not bFirst):
					screen.attachLabel(panelName, "", localText.getText("TXT_KEY_OR", ()))
				else:
					bFirst = False
				screen.attachImageButton( panelName, "", gc.getTechInfo(eTech).getButton(), GenericButtonSizes.BUTTON_SIZE_CUSTOM, WidgetTypes.WIDGET_PEDIA_JUMP_TO_REQUIRED_TECH, eTech, j, False )					
			
		if len(szRightDelimeter) > 0:
			screen.attachLabel(panelName, "", szRightDelimeter)


	def placeUnits(self):

		screen = self.top.getScreen()
		panelName = self.top.getNextWidgetName()
		szTitleDesc = localText.getText("TXT_KEY_PEDIA_UNITS_ENABLED", ())
		screen.addPanel( panelName, szTitleDesc, "", false, true, self.X_UNIT_PANE, self.Y_UNIT_PANE, self.W_UNIT_PANE, self.H_UNIT_PANE, PanelStyles.PANEL_STYLE_BLUE50 )
		screen.attachLabel(panelName, "", "  ")
		
#		for j in range( gc.getNumUnitClassInfos() ):
#			eLoopUnit = gc.getCivilizationInfo(gc.getGame().getActiveCivilizationType()).getCivilizationUnits(j)
		for eLoopUnit in range(gc.getNumUnitInfos()):
			if (eLoopUnit != -1):
				if (isTechRequiredForUnit(self.iTech, eLoopUnit)):
					screen.attachImageButton( panelName, "", gc.getUnitInfo(eLoopUnit).getButton(), GenericButtonSizes.BUTTON_SIZE_CUSTOM, WidgetTypes.WIDGET_PEDIA_JUMP_TO_UNIT, eLoopUnit, 1, False )

	def placeBuildings(self):

		screen = self.top.getScreen()
		panelName = self.top.getNextWidgetName()
		szTitleDesc = localText.getText("TXT_KEY_PEDIA_BUILDINGS_ENABLED", ())
		screen.addPanel( panelName, szTitleDesc, "", false, true, self.X_BUILDING_PANE, self.Y_BUILDING_PANE, self.W_BUILDING_PANE, self.H_BUILDING_PANE, PanelStyles.PANEL_STYLE_BLUE50 )
		screen.attachLabel(panelName, "", "  ")
		
#		for j in range(gc.getNumBuildingClassInfos()):
#			eLoopBuilding = gc.getCivilizationInfo(gc.getGame().getActiveCivilizationType()).getCivilizationBuildings(j)
		for eLoopBuilding in range(gc.getNumBuildingInfos()):
			if (eLoopBuilding != -1):
				if (isTechRequiredForBuilding(self.iTech, eLoopBuilding)):
					screen.attachImageButton( panelName, "", gc.getBuildingInfo(eLoopBuilding).getButton(), GenericButtonSizes.BUTTON_SIZE_CUSTOM, WidgetTypes.WIDGET_PEDIA_JUMP_TO_BUILDING, eLoopBuilding, 1, False )
						
		for eLoopProject in range(gc.getNumProjectInfos()):
			if (isTechRequiredForProject(self.iTech, eLoopProject)):
				screen.attachImageButton( panelName, "", gc.getProjectInfo(eLoopProject).getButton(), GenericButtonSizes.BUTTON_SIZE_CUSTOM, WidgetTypes.WIDGET_PEDIA_JUMP_TO_PROJECT, eLoopProject, 1, False )

	def placeOther(self):

		screen = self.top.getScreen()
		panelName = self.top.getNextWidgetName()
		szTitleDesc = localText.getText("TXT_KEY_PEDIA_CATEGORY_IMPROVEMENT", ())
		screen.addPanel( panelName, szTitleDesc, "", false, true, self.X_OTHER_PANE, self.Y_OTHER_PANE, self.W_OTHER_PANE, self.H_OTHER_PANE, PanelStyles.PANEL_STYLE_BLUE50 )
		screen.attachLabel(panelName, "", "  ")

		# Promotions					  
		for j in range(gc.getNumPromotionInfos()):
			if (gc.getPromotionInfo(j).getTechPrereq() == self.iTech):
				screen.attachImageButton( panelName, "", gc.getPromotionInfo(j).getButton(), GenericButtonSizes.BUTTON_SIZE_CUSTOM,
							  WidgetTypes.WIDGET_PEDIA_JUMP_TO_PROMOTION, j, 1, False )

		# Improvements
		for j in range(gc.getNumBuildInfos()):
				bTechFound = 0
				if (gc.getBuildInfo(j).getTechPrereq() == -1):
					bTechFound = 0
					for k in range(gc.getNumFeatureInfos()):
						if (gc.getBuildInfo(j).getFeatureTech(k) == self.iTech):
							bTechFound = 1
				else:
					if (gc.getBuildInfo(j).getTechPrereq() == self.iTech):
						bTechFound = 1

				if (bTechFound == 1):
					if (gc.getBuildInfo(j).getImprovement() == -1):
						screen.attachImageButton( panelName, "", gc.getBuildInfo(j).getButton(), GenericButtonSizes.BUTTON_SIZE_CUSTOM, WidgetTypes.WIDGET_HELP_IMPROVEMENT, j, 1, False )
					else:
						screen.attachImageButton( panelName, "", gc.getBuildInfo(j).getButton(), GenericButtonSizes.BUTTON_SIZE_CUSTOM, WidgetTypes.WIDGET_PEDIA_JUMP_TO_IMPROVEMENT, gc.getBuildInfo(j).getImprovement(), 1, False )

		# Bonuses
		for j in range(gc.getNumBonusInfos()):
			if (gc.getBonusInfo(j).getTechReveal() == self.iTech):
				screen.attachImageButton( panelName, "", gc.getBonusInfo(j).getButton(), GenericButtonSizes.BUTTON_SIZE_CUSTOM,
							  WidgetTypes.WIDGET_PEDIA_JUMP_TO_BONUS, j, 1, False )

		# Civics
		for j in range(gc.getNumCivicInfos()):
			if (gc.getCivicInfo(j).getTechPrereq() == self.iTech):
				screen.attachImageButton( panelName, "", gc.getCivicInfo(j).getButton(), GenericButtonSizes.BUTTON_SIZE_CUSTOM,
							  WidgetTypes.WIDGET_PEDIA_JUMP_TO_CIVIC, j, 1, False )
		

	def placeSpecial(self):

		screen = self.top.getScreen()
		panelName = self.top.getNextWidgetName()
		szTitleDesc = localText.getText("TXT_KEY_PEDIA_SPECIAL_ABILITIES", ())
		screen.addPanel( panelName, szTitleDesc, "", true, false, self.X_SPECIAL_PANE, self.Y_SPECIAL_PANE, self.W_SPECIAL_PANE, self.H_SPECIAL_PANE, PanelStyles.PANEL_STYLE_BLUE50 )
		
		listName = self.top.getNextWidgetName()
		
		szSpecialText = CyGameTextMgr().getTechHelp(self.iTech, True, False, False, False, -1)
		screen.addMultilineText(listName, szSpecialText, self.X_SPECIAL_PANE+5, self.Y_SPECIAL_PANE+5, self.W_SPECIAL_PANE-10, self.H_SPECIAL_PANE-10, WidgetTypes.WIDGET_GENERAL, -1, -1, CvUtil.FONT_LEFT_JUSTIFY)	
		

	def placeQuote(self):

		screen = self.top.getScreen()
		panelName = self.top.getNextWidgetName()
		screen.addPanel(panelName, "", "", true, true,
			self.X_QUOTE_PANE, self.Y_QUOTE_PANE, self.W_QUOTE_PANE, self.H_QUOTE_PANE, PanelStyles.PANEL_STYLE_BLUE50)

		szQuote = self.techInfo.getQuote() + "\n\n" + self.techInfo.getCivilopedia()
		szQuoteTextWidget = self.top.getNextWidgetName()
		screen.addMultilineText( szQuoteTextWidget, szQuote, self.X_QUOTE, self.Y_QUOTE, self.W_QUOTE, self.H_QUOTE, WidgetTypes.WIDGET_GENERAL, -1, -1, CvUtil.FONT_LEFT_JUSTIFY)
		
	def placeLinks(self):

		self.top.placeLinks(CivilopediaPageTypes.CIVILOPEDIA_PAGE_TECH)
		self.top.placeTechs(self.iTech)

								
	# Will handle the input for this screen...
	def handleInput (self, inputClass):
		return 0