## Sid Meier's Civilization 4
## Copyright Firaxis Games 2005
##
## Sevopedia 
##   sevotastic.blogspot.com
##   sevotastic@yahoo.com
##


from CvPythonExtensions import *
import CvUtil
import ScreenInput
import CvScreenEnums

# globals
gc = CyGlobalContext()
ArtFileMgr = CyArtFileMgr()
localText = CyTranslator()

class CvPediaUnitChart:
	"Civilopedia Screen for Unit Combat Groups"

	def __init__(self, main):
		self.iGroup = -1
		self.top = main
		
		self.X_UNITS = self.top.X_PEDIA_PAGE + 35
		self.Y_UNITS = self.top.Y_PEDIA_PAGE + 15
		self.W_UNITS = 560
		self.H_UNITS = 635
		self.DY_UNITS = 40
		self.Y_TEXT_MARGIN = 6
		
		self.X_TABLE_MARGIN = 40
		self.Y_TABLE_MARGIN = 20
		
		self.ICON_SIZE = 72
		self.ICON_BORDER = 20
		self.X_ICON = self.X_UNITS + (self.W_UNITS - self.ICON_SIZE) / 2
		self.Y_ICON = self.Y_UNITS + self.ICON_BORDER
		self.Y_PANEL_OFFSET = self.ICON_SIZE + self.ICON_BORDER + self.Y_TABLE_MARGIN
		
		self.X_PANEL = self.X_UNITS + self.X_TABLE_MARGIN
		self.Y_PANEL = self.Y_UNITS + self.Y_PANEL_OFFSET
		self.W_PANEL = self.W_UNITS - (self.X_TABLE_MARGIN * 2)
		self.H_PANEL = self.H_UNITS - (self.Y_TABLE_MARGIN * 2) - self.Y_PANEL_OFFSET
		
		self.X_TABLE = self.X_PANEL
		self.Y_TABLE = self.Y_PANEL + 5
		self.W_TABLE = self.W_PANEL
		self.H_TABLE = self.H_PANEL - 10
		
		

	# Screen construction function
	def interfaceScreen(self, iGroup):	
			
		self.iGroup = iGroup
	
		self.top.deleteAllWidgets()						
							
		screen = self.top.getScreen()
		
		bNotActive = (not screen.isActive())
		if bNotActive:
			self.top.setPediaCommonWidgets()

		# Header...
		if (self.iGroup >= gc.getNumUnitCombatInfos()):
			szHeader = localText.getText("TXT_KEY_PEDIA_ALL_UNITS", ())
		else:
			szHeader = gc.getUnitCombatInfo(self.iGroup).getDescription()
		szHeader = u"<font=4b>" + szHeader.upper() + u"</font>"
		szHeaderId = self.top.getNextWidgetName()
		screen.setLabel(szHeaderId, "Background", szHeader, CvUtil.FONT_CENTER_JUSTIFY, self.top.X_SCREEN, self.top.Y_TITLE, 0, FontTypes.TITLE_FONT, WidgetTypes.WIDGET_GENERAL, -1, -1)
		
		# Top
		screen.setText(self.top.getNextWidgetName(), "Background", self.top.MENU_TEXT, CvUtil.FONT_LEFT_JUSTIFY, self.top.X_MENU, self.top.Y_MENU, 0, FontTypes.TITLE_FONT, WidgetTypes.WIDGET_PEDIA_MAIN, CivilopediaPageTypes.CIVILOPEDIA_PAGE_UNIT_GROUP, -1)

		if self.top.iLastScreen	!= CvScreenEnums.PEDIA_UNIT_CHART or bNotActive:		
			self.placeLinks()
			self.top.iLastScreen = CvScreenEnums.PEDIA_UNIT_CHART
				
		self.placeUnitTable()
						
	def placeUnitTable(self):
		screen = self.top.getScreen()
		
		panelName = self.top.getNextWidgetName()
		screen.addPanel( panelName, "", "", true, true,
			self.X_UNITS, self.Y_UNITS, self.W_UNITS, self.H_UNITS, PanelStyles.PANEL_STYLE_BLUE50 )
		
		panelName2 = self.top.getNextWidgetName()
		screen.addPanel( panelName2, "", "", true, true,
                                 self.X_PANEL, self.Y_PANEL, self.W_PANEL, self.H_PANEL, PanelStyles.PANEL_STYLE_BLUE50 )
                
                # Icon
                screen.addDDSGFC(self.top.getNextWidgetName(), gc.getUnitCombatInfo(self.iGroup).getButton(),
		    self.X_ICON, self.Y_ICON, self.ICON_SIZE, self.ICON_SIZE, WidgetTypes.WIDGET_GENERAL, -1, -1 )
                
		szTable = self.top.getNextWidgetName()
		screen.addTableControlGFC(szTable, 4,
			self.X_TABLE, self.Y_TABLE, self.W_TABLE, self.H_TABLE, True, False, 32,32, TableStyles.TABLE_STYLE_EMPTY)
		screen.enableSort(szTable)

		iFirstColWidth = int(self.W_TABLE * (7 / 19.0))
		screen.setTableColumnHeader(szTable, 0, "", iFirstColWidth)
		iColWidth = int(self.W_TABLE * (4 / 19.0))
		screen.setTableColumnHeader(szTable, 1, u"%c" % CyGame().getSymbolID(FontSymbols.STRENGTH_CHAR), iColWidth)
		screen.setTableColumnHeader(szTable, 2, u"%c" % CyGame().getSymbolID(FontSymbols.MOVES_CHAR), iColWidth)
		screen.setTableColumnHeader(szTable, 3, u"%c" % gc.getYieldInfo(YieldTypes.YIELD_PRODUCTION).getChar(), iColWidth)
					
		# count units in this group
		nUnits = 0
		iMaxTypes = gc.getNumUnitCombatInfos()
		for j in range(gc.getNumUnitInfos()):
			if (self.iGroup == gc.getUnitInfo(j).getUnitCombatType() or self.iGroup == iMaxTypes):
				nUnits += 1

		dy = self.DY_UNITS
		yTextMargin = self.Y_TEXT_MARGIN
		if (self.iGroup == iMaxTypes):
			dy = self.DY_UNITS/2
			yTextMargin = 0

		# initially sort Units by strength
		i = 0
		unitsList=[(0,0,0,0,0)]*nUnits
		for j in range(gc.getNumUnitInfos()):
			if (self.iGroup == gc.getUnitInfo(j).getUnitCombatType() or self.iGroup == iMaxTypes):
				if (gc.getUnitInfo(j).getProductionCost() < 0):
					szCost = localText.getText("TXT_KEY_NON_APPLICABLE", ())
				else:
					szCost = unicode(gc.getUnitInfo(j).getProductionCost())# + u"%c" % gc.getYieldInfo(YieldTypes.YIELD_PRODUCTION).getChar()
				unitsList[i] = (gc.getUnitInfo(j).getCombat(), gc.getUnitInfo(j).getMoves(), szCost, gc.getUnitInfo(j).getDescription(), j)
				i += 1
		unitsList.sort()

		for i in range(nUnits):			
			iRow = screen.appendTableRow(szTable)
			screen.setTableText(szTable, 0, iRow, u"<font=3>" + unitsList[i][3] + u"</font>", "", WidgetTypes.WIDGET_PEDIA_JUMP_TO_UNIT, unitsList[i][4], 1, CvUtil.FONT_LEFT_JUSTIFY)	
			screen.setTableInt(szTable, 1, iRow, u"<font=3>" + unicode(unitsList[i][0]) + u"</font>", "", WidgetTypes.WIDGET_GENERAL, -1, -1, CvUtil.FONT_LEFT_JUSTIFY)
			screen.setTableInt(szTable, 2, iRow, u"<font=3>" + unicode(unitsList[i][1]) + u"</font>", "", WidgetTypes.WIDGET_GENERAL, -1, -1, CvUtil.FONT_LEFT_JUSTIFY)
			screen.setTableInt(szTable, 3, iRow, u"<font=3>" + unicode(unitsList[i][2]) + u"</font>", "", WidgetTypes.WIDGET_GENERAL, -1, -1, CvUtil.FONT_LEFT_JUSTIFY)

	def placeLinks(self):

		self.top.placeLinks(CivilopediaPageTypes.CIVILOPEDIA_PAGE_UNIT_GROUP)
		self.top.placeUnitGroups(self.iGroup)

		
	# Will handle the input for this screen...
	def handleInput (self, inputClass):
		return 0

